//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 2000  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// KeyingStatistics class - counts hits, misses, timings, and calculates
// WPM values when requested
//

/**
 * Counts keystroke hits, misses, timings, and calculates WPM values
 * when requested.
 */
public class KeyingStatistics {

	// statistics for the count of times a key is hit, how
	// long it takes to hit, and how many were missed
	private int	struck_count;//	= 0;
	private int	missed_count;//	= 0;
	private int	ms_expended;//	= 0;


	/**
	 * Register a successful hit.
	 */
	public synchronized void struck( int msec ) {
		struck_count++;
		ms_expended += msec;
	}


	/**
	 * Register a miss.
	 */
	public synchronized void missed( int msec ) {
		missed_count++;
		ms_expended += msec;
	}


	/**
	 * Reset the statistics set to the initial state.
	 */
	public synchronized void reset() {
		struck_count	= 0;
		missed_count	= 0;
		ms_expended	= 0;
	}


	/*
	 * Return the hit count of a key, miss count, and time expended.
	 */
	public synchronized int	get_strikes()	{ return ( struck_count ); }
	public synchronized int	get_misses()	{ return ( missed_count ); }
	public synchronized int	get_ms_expended(){ return ( ms_expended ); }

	/**
	 * Indicate if this set of statistics contains anything useful.
	 */
	public synchronized boolean has_statistics() {
		return ( ms_expended > 0 );
	}


	/**
	 * Calculate % accuracy from the information we have recorded.
	 */
	public synchronized int get_pc_accuracy() {
		if ( struck_count + missed_count > 0 )
			return (( struck_count * 100 )
					/ ( struck_count + missed_count ));
		else
			return ( 0 );
	}

	/**
	 * Calculate % errors from the information we have recorded.
	 */
	public synchronized int get_pc_errors() {
		if ( struck_count + missed_count > 0 )
			return ( 100 - get_pc_accuracy() );
		else
			return ( 0 );
	}


	/**
	 * Calculate the WPM from hits and timings; adjust for errors
	 * if requested.
	 */
	public synchronized int get_WPM( boolean adjusted ) {

		// if any time at all registered OK to continue
		if ( ms_expended > 0 ) {

			// calculate effective elapsed time
			float secs = (float)ms_expended / 1000.0f;
			float mins = secs / 60.0f;

			// calculate effective words (5 chars) - here we
			// include misses, since even though they were
			// misses something was struck; we'll account for
			// misses in adjusted WPM later
			float effective_words = (float)( struck_count
						+ missed_count) / 5.0f;

			// apply a word penalty for each miss if adjusting,
			// and clamp -ve word counts to zero (can't really get
			// any lower)
			if ( adjusted ) {
				effective_words -= (float)missed_count;
				if ( effective_words < 0.0f )
					effective_words = 0.0f;
			}

			// find WPM, and take it as an integer, with rounding
			int wpm = (int)( ( effective_words / mins ) + 0.5f );
			return ( wpm );
		}
		else
			// no timings taken
			return ( 0 );
	}


	/**
	 * Method to combine this stats set with another.
	 */
	public synchronized void add( KeyingStatistics kbstats ) {
		this.struck_count	+= kbstats.get_strikes();
		this.missed_count	+= kbstats.get_misses();
		this.ms_expended	+= kbstats.get_ms_expended();
	}


	/**
	 * Printout method.
	 */
	public synchronized String toString() {
		return ( getClass().getName() + "{struck_count=" + struck_count
				+ ",missed_count=" + missed_count
				+ ",ms_expended=" + ms_expended
				+ ",get_pc_accuracy()=" + get_pc_accuracy()
				+ ",get_pc_errors()=" + get_pc_errors()
				+ ",get_WPM(false)=" + get_WPM( false )
				+ ",get_WPM(true)=" + get_WPM( true ) + "}" );
	}


	/**
	 * Class test code.
	 */
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		KeyingStatistics g = new KeyingStatistics();
		System.out.println( "initial : " + g );
		g.struck( 200 );
		System.out.println( "1@200mS : " + g );
		KeyingStatistics g2 = new KeyingStatistics();
		g2.struck( 300 ); g2.missed( 100 );
		g.add( g2 );
		System.out.println( "+1@300mS/1@100mS : " + g );

		for ( int i = 0; i < 1000; i++ )
			g.struck( i );
		for ( int i = 0; i < 100; i++ )
			g.missed( i );
		System.out.println( "full : " + g );
	}
}
